/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2006, Anthony Minessale
 *
 * Anthony Minessale II <anthmhct@yahoo.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! \file
 *
 * \brief Segfault Handler
 *
 * \author Anthony Minessale II <anthmhct@yahoo.com>
 */

#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 39497 $");

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>

#include "asterisk/channel.h"
#include "asterisk/module.h"
#include "asterisk/cli.h"
#include "asterisk/options.h"

static char *EXEC = NULL;

static int handle_SIGSEGV(int sig)
{
	system(EXEC);
	abort();
    return 0;
}

static int unload_module(void *mod)
{
	return -1;
}

static int load_config(void)
{
	int res = 0;
	struct ast_config *cfg;
	char *entry;

	if (EXEC) {
		free(EXEC);
	}
	EXEC = NULL;
	
	if (!(cfg = ast_config_load("segfault.conf"))) {
		return res;
	}
	
	for (entry = ast_category_browse(cfg, NULL); entry != NULL; entry = ast_category_browse(cfg, entry)) {
		if (!strcasecmp(entry, "general")) {
			struct ast_variable *vp, *var = ast_variable_browse(cfg, entry);
			
			for(vp = var ; vp ; vp = vp->next) {
				if (!strcasecmp(vp->name, "exec")) {
					EXEC = strdup(vp->value);
					if (option_verbose > 2) {
						ast_verbose(VERBOSE_PREFIX_3 "Setting Segfault Command to [%s]\n", EXEC);
					}
					res = 1;
					goto done;
				}
			}
			break;
		}
	}
 done:

	ast_config_destroy(cfg);

	return res;
	
}

static int reload(void *mod)
{

	if (load_config()) {
		signal(SIGSEGV, (void *) handle_SIGSEGV);
	} else {
		signal(SIGSEGV, SIG_DFL);
	}	
	
	return 0;
}

static int load_module(void *mod)
{
	return reload(mod);
}

static const char *description(void)
{
	return "Segfault Handler";
}

static const char *key(void)
{
	return ASTERISK_GPL_KEY;
}

STD_MOD(MOD_0 | NO_USECOUNT | NO_UNLOAD, reload, NULL, NULL);
